@@ -34,9 +34,11 @@ if 0: ######## INIT
            'imp importlib importlib.abc importlib.util ' #### import hook
            'locale') ## PYTHON BUG
   class PseudoSugar(ast.NodeVisitor, importlib.abc.Finder, importlib.abc.PyLoader):
+    # py3to2 = None
+
     ## convenience function
     @staticmethod
-    def exec(ss, globals, locals = None, fpath = '<file>'): exec( PseudoSugar().compile(ss, fpath, 'exec'), globals, locals )
+    def exec(ss, globals, locals = None, fpath = '<file>'): exec( PseudoSugar.compile(ss, fpath, 'exec'), globals, locals )
 
     ## recursively print nodes in ast object for debugging
     @staticmethod
@@ -46,30 +48,6 @@ if 0: ######## INIT
       for aa in ast.iter_child_nodes(node): ss += PseudoSugar.debugnode(aa, depth = depth + ' ')
       return ss
 
-    # ## parse string into legal python syntax
-    # @staticmethod
-    # def parse_str(ss,
-                  # rgx_pseudometh2 = re.compile(' \.\.(\w[\w. ]*\()'),
-                  # rgx_pseudometh3 = re.compile(' \.\.\.(\w[\w. ]*\()'),
-                  # rgx_pseudometh4 = re.compile(' \.\.\.\.(\w[\w. ]*\()'),
-                  # rgx_prefixop_print = re.compile('(^|\W)print@@ '),
-                  # rgx_prefixop = re.compile('(\))<<<< '),
-                  # rgx_postfixop = re.compile(' >>>>(\w[\w. ]*\()'),
-                  # ):
-      # # print(ss[:1024])
-      # # ss = PseudoSugar.magic.sub('## \\1', ss, 1)
-      # # print(ss[:1024])
-      # ss = rgx_pseudometh2.sub(' .__pseudometh2__.\\1', ss) ## parse pseudometh2 syntax
-      # ss = rgx_pseudometh3.sub(' .__pseudometh3__.\\1', ss) ## parse pseudometh3 syntax
-      # ss = rgx_pseudometh4.sub(' .__pseudometh4__.\\1', ss) ## parse pseudometh4 syntax
-      # ss = rgx_prefixop_print.sub('\\1print()<<<< ', ss)
-      # ss = rgx_prefixop.sub('\\1.__prefixop4__, ', ss)
-      # ss = rgx_postfixop.sub(' ,__postfixop4__.\\1', ss)
-      # return ss
-
-    # @staticmethod
-    # def unparse_str(ss): return ss.replace(',__postfixop4__.', '>>>>').replace('.__prefixop4__,', '<<<<').replace('print()<<<<', 'print@@').replace('.__pseudometh4__.', '....').replace('.__pseudometh3__.', '...').replace('.__pseudometh2__.', '..')
-
     ## parse string into legal python syntax
     @staticmethod
     def parse_str(ss,
@@ -81,10 +59,7 @@ if 0: ######## INIT
                   rgx_prefixop = re.compile('(\))<<<< '),
                   rgx_postfixop = re.compile(' >>>>(\w[\w. ]*\()'),
                   ):
-      # print(ss[:1024])
-      # ss = PseudoSugar.magic.sub('## \\1', ss, 1)
-      # print(ss[:1024])
-      ss = rgx_pseudometh2.sub(' .__pseudometh2__.\\1', ss) ## parse pseudometh2 syntax
+      ss = rgx_pseudometh2.sub(' .__pseudometh2__.\\1', ss)
       ss = rgx_pseudometh3.sub(' .__pseudometh3__.\\1', ss) ## parse pseudometh3 syntax
       ss = rgx_pseudometh4.sub(' .__pseudometh4__.\\1', ss) ## parse pseudometh4 syntax
       ss = rgx_applyhead.sub('\\1()<<<< ', ss)
@@ -93,9 +68,6 @@ if 0: ######## INIT
       ss = rgx_postfixop.sub(' ,__postfixop4__.\\1', ss)
       return ss
 
-    # @staticmethod
-    # def unparse_str(ss): return ss.replace(',__postfixop4__.', '>>>>').replace('.__prefixop4__,', '<<<<').replace('()<<<<', '@@').replace('.__pseudometh4__.', '....').replace('.__pseudometh3__.', '...').replace('.__pseudometh2__.', '..')
-
     @staticmethod
     def unparse_str(ss): return ss.replace(' ,__postfixop4__.', ' >>>>').replace('.__prefixop4__, ', '<<<< ').replace('()<<<< ', '@@ ').replace(' >>>>()', ' @@').replace(' .__pseudometh4__.', ' ....').replace(' .__pseudometh3__.', ' ...').replace(' .__pseudometh2__.', ' ..')
 
@@ -105,21 +77,11 @@ if 0: ######## INIT
       except: lineno, col_offset = self.visit_lineno(self.node); raise SyntaxError('invalid pseudosugar syntax', (fpath, lineno, col_offset, '\n'.join(self.unparse_str(ss).split('\n')[max(lineno - 4, 0): lineno])))
       assert not self.parents, self.parents; return node
 
-    # @staticmethod
-    # def compile(ss, fpath, mode, flags = 0, dont_inherit = 0):
-      # try:
-        # if not PseudoSugar.magic.search(ss): return builtins.__pseudocompile__(ss, fpath, mode, flags, dont_inherit)
-      # except SyntaxError: pass
-      # node = PseudoSugar().parse_ast(ss, fpath, mode)
-      # if flags & ast.PyCF_ONLY_AST: return node
-      # return builtins.__pseudocompile__(node, fpath, mode, flags, dont_inherit)
-    # if not hasattr(builtins, '__pseudocompile__'): builtins.__pseudocompile__ = builtins.compile
-
     @staticmethod
     def compile(ss, fpath, mode, flags = 0, dont_inherit = 0):
-      try:
-        if not PseudoSugar.hasmagic(ss): return builtins.__pseudocompile__(ss, fpath, mode, flags, dont_inherit)
-      except SyntaxError: pass
+      if not PseudoSugar.hasmagic(ss):
+        try: return builtins.__pseudocompile__(ss, fpath, mode, flags, dont_inherit)
+        except SyntaxError: pass
       node = PseudoSugar().parse_ast(ss, fpath, mode)
       if flags & ast.PyCF_ONLY_AST: return node
       return builtins.__pseudocompile__(node, fpath, mode, flags, dont_inherit)
@@ -713,51 +675,32 @@ if 1: ######## UTIL
     def hasmagic(ss): return Py3to2.magic.search(ss)
 
     @staticmethod
-    def py3to2(ss):
+    def py3to2(ss, rgx_format = re.compile('([^.])\.format\(')):
       self = Py3to2
       if not self.hasmagic(ss): raise SyntaxError('py3to2 requires source\n<{!r}>\nto have magic line "## from __future__ import py3to2"'.format(ss[:64]))
-      aa = uniquestr(ss); ss = self.magic.sub(aa, ss, 1); ss = self().unparse_ast(compile(ss, '', 'exec', ast.PyCF_ONLY_AST)); ss = ss.replace(aa, 'exec({!r}, globals())'.format('''from __future__ import absolute_import, division, generators, nested_scopes, with_statement\ndef {aa}():
-      import sys
+      ss = rgx_format.sub('\\1 ..__format25__(', ss)
+      aa = uniquestr(ss); ss = self.magic.sub(aa, ss, 1); ss = self().unparse_ast@@ compile(ss, '', 'exec', ast.PyCF_ONLY_AST); ss = ss.replace(aa, 'exec({!r}, globals())'.format('''from __future__ import absolute_import, division, generators, nested_scopes, with_statement\ndef {aa}():
+      import os, sys, string, random
       for aa in 'builtins:__builtin__ urllib.request:urllib2 urllib.error:urllib2 urllib.parse:urlparse'.split(' '):
         if aa in sys.modules: continue
         aa, bb = aa.split(':'); sys.modules[aa] = __import__(bb)
         if '.' in aa: aa, cc = aa.split('.'); setattr(__import__(aa), cc, __import__(bb))
       import cgi, urlparse; urlparse.parse_qs = cgi.parse_qs
+      def uniquestr(ss, kwd = 'qjzx', alphanum = '_0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'):
+        while kwd in ss: kwd += random.choice(alphanum)
+        return kwd
+      global __format25__; import string; format_rgx = [(aa, '__format25__' + str(ii)) for ii, aa in enumerate('{{{{ }}}} $'.split(' '))]
+      def __format25__(ss, *args, **kwds):
+        for aa, bb in format_rgx: ss = ss.replace(aa, bb)
+        if args: ss = ss.replace('{{}}', '%s') % args
+        if kwds: ss = string.Template(ss.replace('{{', '${{')).substitute(**kwds)
+        for aa, bb in format_rgx:
+          if aa in '{{{{}}}}': aa = aa[0]
+          ss = ss.replace(bb, aa)
+        return ss
+      __format25__.format_rgx = format_rgx
       \n{aa}(); del {aa}'''.format(aa = aa)), 1); return ss
 
-    # name25 = dict(aa.split(':') for aa in 'print:__print25__'.split(' '))
-
-    # def visit(self, node, ss = ''):
-      # type = node.__class__
-      # # ## pre
-      # # if type is ast.ExceptHandler: ss = 'except {}{}:{}' ..self.format(node.type if node.type else '', '({})'.format(node.name) if node.name else '', node.body, indent = 1)
-      # ss = super().visit(node, ss)
-      # ## post
-      # if type is ast.Call:
-        # if isinstance(node.func, ast.Name) and node.func.id == 'print': ss = ss.replace('(', ' ', 1) ..sreverse().replace(')', '', 1) ..sreverse() ## print statement
-      # # if type is ast.Call:
-        # # # if super():
-        # # # if self.visit(node.func) == 'print': return super().visit(node).replace('(', ' ', 1) ..sreverse().replace(')', '', 1) ..sreverse() ## print statement
-      # # return super().visit(node, ss)
-      # # elif type is ast.ExceptHandler: ss = 'except {}{}:{}' ..self.format(node.type if node.type else '', '({})'.format(node.name) if node.name else '', node.body, indent = 1)
-      # return ss
-
-    # def visit_pre(self, node):
-      # ss = ''; type = node.__class__
-      # if type is ast.Call:
-        # if isinstance(node.func, ast.Name) and node.func.id == 'print':
-          # # ss = ss.replace('(', ' ', 1) ..sreverse().replace(')', '', 1) ..sreverse() ## print statement
-          # args = list(self.generic_visit(node.args + node.keywords))
-          # if node.starargs: args.append('*' + self.visit(node.starargs))
-          # if node.kwargs: args.append('**' + self.visit(node.kwargs))
-          # ss = '{} {}' ..self.format(node.func, args, join = ',')
-      # # if type is ast.Call:
-        # # # if super():
-        # # # if self.visit(node.func) == 'print': return super().visit(node).replace('(', ' ', 1) ..sreverse().replace(')', '', 1) ..sreverse() ## print statement
-      # # return super().visit(node, ss)
-      # elif type is ast.ExceptHandler: ss = 'except {}{}:{}' ..self.format(node.type if node.type else '', '({})'.format(node.name) if node.name else '', node.body, indent = 1)
-      # return ss
-
     @staticmethod
     def test():
       self = Py3to2()
@@ -802,15 +745,14 @@ if 1: ######## BUILD
 
   htmlserver/_main.cpp
   htmlserver/lucida06x10.bmp
-  htmlserver/mario.png 
+  htmlserver/mario.png
 
-  htmlserver/htmlserver.py
   htmlserver/index.html
 
   htmlserver/_README.py'''.replace(' ', '')
   _DIST = Namespace(
     name = 'htmlserver',
-    version = '2010.02.06.appengine',
+    version = '2010.02.07.appengine',
     author = 'kai zhu',
     author_email = 'kaizhu256@gmail.com',
     license = 'gpl',
@@ -825,7 +767,7 @@ if 1: ######## BUILD
     ext_modules = _EXTENSION,
     classifiers = [
     'Development Status :: 3 - Alpha',
-    # 'Development Status :: 4 - Beta',
+    ## 'Development Status :: 4 - Beta',
     'Intended Audience :: Developers',
     'Intended Audience :: End Users/Desktop',
     'Intended Audience :: Science/Research',
@@ -847,9 +789,9 @@ if 1: ######## BUILD
     'Topic :: Software Development :: Code Generators',
     'Topic :: Software Development :: Libraries',
     'Topic :: Software Development :: Libraries :: Python Modules',
-    # 'Topic :: System :: Emulators',
-    # 'Topic :: System :: Shells',
-    # 'Topic :: System :: System Shells',
+    ## 'Topic :: System :: Emulators',
+    ## 'Topic :: System :: Shells',
+    ## 'Topic :: System :: System Shells',
     'Topic :: Utilities',
     ])
 
@@ -885,11 +827,20 @@ if 1: ######## BUILD
       user_options = [('name=', None, 'server name'),
                       ('version=', None, 'version'),
                       ('api-version=', None, 'api version'),
-                      # ('google-appengine-path=', None, 'google_appengine path'),
                       ('serve=', None, ''),
+                      ('update', None, ''),
                      ]
 
-      def run(self): kwds = dict_append(dict((aa, getattr(self, aa)) for aa in self.subcommands if getattr(self, aa)), name = 'htmlserver', version = '1', api_version = '1', serve = '1'); import htmlserver.htmlserver; htmlserver.htmlserver.serve_forever(setup = True, **kwds)
+      # def run(self): kwds = dict_append(dict((aa, getattr(self, aa)) for aa in self.subcommands if getattr(self, aa)), name = 'htmlserver', version = '1', api_version = '1', serve = '1'); import htmlserver.htmlserver; htmlserver.htmlserver.serve_forever(setup = True, **kwds)
+
+      # def run(self): kwds = dict_append(dict((aa, getattr(self, aa)) for aa in self.subcommands if getattr(self, aa)), name = 'htmlserver', version = '1', api_version = '1', serve = '1'); exec@@ re.search('(?s)\nif 0: ######## HTMLSERVER.*?\n## END\n', PseudoSugar.compile@@ open('htmlserver/main.py').read()).group().replace('\nif 0', '\nif 1', 1), globals(); serve_forever(setup = True, **kwds)
+
+      def run(self):
+        kwds = dict_append(dict((aa, getattr(self, aa)) for aa in self.subcommands if getattr(self, aa)), name = 'htmlserver', version = '1', api_version = '1', serve = '1')
+        open('htmlserver/main.py').read() ...re.search('(?s)\nif 0: ######## HTMLSERVER.*?\n## from __future__ import py3to2\n').group().replace('\nif 0:', '\nif 1:', 1) ..PseudoSugar.exec(globals())
+        # ss = re.search@@ '(?s)\nif 0: ######## HTMLSERVER.*?\n## END\n', open('htmlserver/main.py').read()
+        # ss = ss.group()
+        serve_forever(setup = True, **kwds)
 
     def __init__(self, kwds, dev = Dev, htmlserver = HtmlServer): super().__init__(kwds); self.cmdclass['dev'] = dev; self.cmdclass['htmlserver'] = htmlserver; _MODULE.BUILD = self
 
@@ -923,7 +874,6 @@ if 1: ######## BUILD
           else: system( '  cp -a {} {}'.format(aa, bb) ) ..print() ## direct copy
         system( 'python3.1 setup.py {}'.format(' '.join(sys.argv[3:])), block = None ); exit()
       if self.doc: system( 'python3.1 -c "import htmlserver; help(htmlserver)"', block = None )
-      # if self.htmlserver: import htmlserver.htmlserver; htmlserver.htmlserver.serve_forever(self.htmlserver); exit()
       if self.pkginfo: self.append_pkginfo()
       if self.quicktest: _MODULE._SETUP = None; reload(_MODULE); print(_MODULE); quicktest()
       if self.sdist_test:
@@ -938,28 +888,10 @@ if 1: ######## BUILD
       if self.test: _MODULE._SETUP = None; reload(_MODULE); quicktest(self.test)
       if self.uninstall: self.run_uninstall(self.uninstall)
 
-    # def run_html(self):
-      # from htmlserver import htmlserver
-      # htmlserver.name = self.name
-      # htmlserver.serve_forever(); exit()
-
     def run_uninstall(self, prefix): cmd = self.get_command_obj('install'); cmd.prefix = prefix; cmd.ensure_finalized(); system( 'rm -Ir {}'.format(os.path.join(cmd.install_lib, 'htmlserver')) ) ..print()
 
     def append_pkginfo(self): self.metadata.long_description += system( 'python3.1 -c "import htmlserver; htmlserver.quicktest(pkginfo = True)"' ); self.metadata.write_pkg_file(open('README', 'w')); pydoc.pager(open('README').read())
 
-    # def append_pkginfo(self):
-      # if 'htmlserver' == 'htmlserver':
-        # # cmd = 'python3.1 setup.py htmlserver --name=myserver\n'
-        # # print@@ system( cmd )
-        # # # self.metadata.long_description += cmd + system( cmd )
-        # cmd = 'python3.1 setup.py htmlserver --name=myserver --serve=0'
-        # # print@@ system( cmd )
-        # self.metadata.long_description += '\n$ {}\n{}'.format@@ cmd, system( cmd + ' 2>&1', debug = None )
-      # else: self.metadata.long_description += system( 'python3.1 -c "import htmlserver; htmlserver.quicktest()"' )
-      # self.metadata.write_pkg_file(open('README', 'w')); pydoc.pager(open('README').read())
-
-    def post_install(self, cmd, **kwds): dpath = cmd.install_lib + 'htmlserver/'; self.get_command_obj('build_ext').force = True; self.run_command('build_ext', rerun = True, install_path = dpath)
-
     ## generate c header
     def c2h(self, src):
       ss = src.split('\n'); hh = ''; macro = {}
@@ -1164,28 +1096,22 @@ if _EXTENSION and not _SETUP: ######## EXTENSION
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
 ## print test log
 @_import('code')
 def quicktest(type = None, pkginfo = None):
-  # print@@ type
-  # exit()
   if not type: type = 'htmlserver' if hasattr(_README, 'htmlserver') else 'numbyte'
-  # if type == 'htmlserver':
-    # # print@@ '''python3.1 setup.py htmlserver --name=myserver
-    # # '''
-    # cmd = 'python3.1 setup.py htmlserver --name=myserver --serve=0'
-    # # print@@ system( cmd )
-    # print@@ '\n$ {}\n{}'.format@@ cmd, system( cmd + ' 2>&1', debug = None )
-    # return
-    # # exit()
-  # if type == 'htmlserver':
-    # cmd = 'python3.1 setup.py htmlserver --name=myserver'
-    # print@@ '\n$ {}\n{}'.format@@ cmd, system( cmd + (' --serve=0 2>&1' if pkginfo else ''), debug = None )
-    # return
-  # if type == 'htmlserver': return 
-    # cmd = 'python3.1 setup.py htmlserver --name=myserver'
-    # print@@ '\n$ {}\n{}'.format@@ cmd, system( cmd + (' --serve=0 2>&1' if pkginfo else ''), debug = None )
-    # return
   type = getattr(_README, type)
   if type == _README.htmlserver: return type.test(pkginfo)
   tt = CodeTree.src(type.test, CodeTree.src(type)); rgx = re.compile('^' + ' ' * tt[1].depth(), re.M)
@@ -1193,3 +1119,186 @@ def quicktest(type = None, pkginfo = None):
   tt.insert(0, '>>> from htmlserver import *\n')
   mm = imp.new_module('test'); console = code.InteractiveConsole(vars(mm))
   for aa in tt: print(aa); console.runsource( aa.replace('>>> ', '').replace('...  ', ' ') + '\n' )
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+if 0: ######## HTMLSERVER
+  ## app.yaml
+  """application: {name}
+  version: {version}
+  api_version: {api_version}
+  runtime: python
+  handlers:
+  - url: /test/admin
+    script: url.py
+    login: admin
+  - url: /test/admin/.*
+    script: url.py
+    login: admin
+  - url: /test.*
+    script: url.py
+    # login: required
+  - url: /(.*\.(css|gif|jpg|png))
+    static_files: \1
+    upload: (.*\.(css|gif|jpg|png))
+  - url: /(url_\w+\.html)
+    static_files: \1
+    upload: /(url_\w+\.html)
+  - url: /.*
+    script: url.py
+  """
+  ## END
+
+  import os, sys, re, signal, subprocess, time
+  def update(KWDS, force = None, modified = 'main.py htmlserver2.py'.split(' '), update = 'app.yaml url.py url2.py'.split(' ')):
+    if not force and all(os.path.exists(aa) for aa in update) and min(os.path.getmtime(aa) for aa in update) >= max(os.path.getmtime(aa) for aa in modified): return
+    ss = open('main.py').read().replace('\n  ', '\n')
+    open('app.yaml', 'w').write@@ re.search('(?s)\n## app.yaml\n"""(.*?)"""\n## END', ss).group(1).format(**KWDS)
+    ss = ss.split('\n## from __future__ import py3to2'); ss = '## from __future__ import py3to2\n' + '\n' * ss[0].count('\n') + ss[1] ## preserve lineno
+    open('url.py', 'w').write@@ Py3to2.py3to2@@ ss
+    open('url2.py', 'w').write@@ Py3to2.py3to2@@ open('htmlserver2.py').read()
+
+  def serve_forever(**KWDS):
+    if not KWDS.get('setup', None): raise Exception('htmlserver.htmlserver.serve_forever can only be called from setup.py')
+    try: subprocess.call@@ ['python2.5', '--version']
+    except: print@@ 'ERROR: python2.5 not found - please install - required by google appengine'; raise
+    if not os.path.exists('google_appengine'):
+      url = 'http://googleappengine.googlecode.com/files/google_appengine_1.3.0.zip'
+      print@@ 'downloading {} ...'.format(url); import urllib.request; fpath, headers = urllib.request.urlretrieve(url); print@@ ' ... done'
+      print@@ 'extracting {} ...'.format@@ os.path.basename@@ url; import zipfile; zipfile.ZipFile(fpath).extractall(); print@@ ' ... done'
+    os.chdir('htmlserver')
+    # for aa in 'htmlserver2.py index2.html'.split(' '):
+      # if not os.path.exists(aa): system( 'touch {}'.format(aa) )
+    if not os.path.exists('htmlserver2.py'): open('htmlserver2.py', 'w').write@@ '## from __future__ import py3to2\n'
+    if not os.path.exists('index2.html'): system( 'touch index2.html' )
+    update(KWDS, force = True)
+    if KWDS.get('update', None): return system( 'python2.5 ../google_appengine/appcfg.py update ../htmlserver', block = None)
+    proc = subprocess.Popen(['python2.5', '../google_appengine/dev_appserver.py', '../htmlserver']) ## server
+    try:
+      if eval(KWDS['serve']):
+        while True: update(KWDS); time.sleep(1) ## update
+      else: time.sleep(1)
+    finally: os.kill(proc.pid, signal.SIGTERM) ## kill server
+
+## python3.1 setup.py htmlserver
+## from __future__ import pseudosugar
+## from __future__ import py3to2
+
+  import os, re, sys
+
+  class appengine(object): ## namespace
+    import cgi
+    from urllib import parse, request
+    from google.appengine.api import images, memcache, users
+    from google.appengine.ext import webapp
+    # from google.appengine.ext.webapp import util, template, RequestHandler
+    from google.appengine.ext.webapp import util, RequestHandler
+    from google.appengine.ext import db
+
+  class URL(appengine.RequestHandler):
+    def __init__(self):
+      self.handlers.append(self)
+      if 1: self.printerr@@ '\nhandlers', self.handlers
+
+    @staticmethod
+    def dispatch():
+      for aa in 'AUTH_TYPE CONTENT_LENGTH CONTENT_TYPE GATEWAY_INTERFACE PATH_INFO PATH_TRANSLATED QUERY_STRING REMOTE_ADDR REMOTE_HOST REMOTE_IDENT REMOTE_USER REQUEST_METHOD SCRIPT_NAME SERVER_NAME SERVER_PORT SERVER_PROTOCOL SERVER_SOFTWARE'.split(' '): setattr@@ URL, aa, os.environ.get(aa, None)
+      URL.handler = url_main; URL.handlers = []; URL.path = URL.PATH_INFO[1:].split('/'); URL.pathi = 0; URL.query = appengine.parse.parse_qs(URL.QUERY_STRING)
+      self = URL(); self.walk_handler()
+      if not self.path[-1]: del self.path[-1] ## del trailing '/'
+      appengine.util.run_wsgi_app@@ appengine.webapp.WSGIApplication(self.handler.map(self.handler), debug = True)
+      if 1: self.printerr@@ 'dispatch', 'path=', self.path, 'pathi=', self.pathi, 'query=', self.query, 'handlers=', self.handlers
+
+    def get(self):
+      dpath = 'http://%s:%s' % (self.SERVER_NAME, self.SERVER_PORT)
+      for aa, bb in sorted@@ vars(self.__class__).iteritems():
+        if type(bb) is type and issubclass(bb, URL): url = '%s/%s' % (dpath, '/'.join(self.path + [aa[4:]])); self.write@@ 'text/html', '<br><a href="%s">%s</a></br>' % (url, url)
+
+    def login(self): login = 'logout' if appengine.users.get_current_user() else 'login'; return '<a href="%s">%s</a>' % (appengine.users.create_logout_url(self.request.uri) if login == 'logout' else appengine.users.create_login_url(self.request.uri), login)
+
+    @staticmethod
+    def map(type): return [('/.*', type)]
+
+    def printerr(self, *args): sys.stderr.write('\t'.join(str(aa) for aa in args)); sys.stderr.write('\n')
+
+    # rgx_comment = re.compile('(?s)<!--.*?-->')
+    # def render(self, name, *args, **kwds): return re.search('(?s)\n<!-- {} -->.*?\n<!-- END -->'.format(name), open('index.html').read() + open('index2.html').read()).group() ...self.rgx_comment.sub('').format(*args, **kwds)
+
+    def render(self, name, *args, **kwds): return re.search('(?s)\n<!-- {} -->.*?\n<!-- END -->'.format(name), open('index.html').read() + open('index2.html').read()).group().format(*args, **kwds)
+
+    def walk_handler(self):
+      if self.pathi >= len(self.path): return
+      for aa, bb in vars(self.handler).iteritems():
+        if type(bb) is type and issubclass(bb, URL) and aa[4:] == self.path[self.pathi]: self.handler = bb; self.pathi += 1; return self.walk_handler()
+
+    def write(self, content_type, *args): self.response.headers['Content-Type'] = content_type; return self.response.out.write@@ '\t'.join(str(aa) for aa in args) + '\n'
+
+  class url_main(URL):
+    class url_test(URL):
+      class url_admin(URL):
+        class url_debug(URL):
+          def get(self):
+            for aa in appengine, self:
+              for bb in dir(aa):
+                if bb[0] != '_': self.write@@ 'text/html', '<li>', bb, repr(getattr(aa, bb))[:64].replace('<', '>'), '</li>'
+
+      class url_login(URL):
+        def get(self): self.write@@ 'text/html', self.login()
+
+      class url_css(URL):
+        def get(self): self.write@@ 'text/html', self.render('url_test_css')
+
+      class url_guestbook(URL):
+        class DataGuestbook(appengine.db.Model):
+          user = appengine.db.UserProperty()
+          date = appengine.db.DateTimeProperty(auto_now_add = True)
+          comment = appengine.db.StringProperty(multiline = True)
+          avatar = appengine.db.BlobProperty()
+
+        def get(self):
+          if 'get_avatar' in self.query:
+            comment = appengine.db.get@@ self.request.get@@ 'get_avatar'
+            if comment.avatar: self.write@@ 'image/png', comment.avatar
+          else:
+            comments = self.DataGuestbook.all().order('-date').fetch(4)
+            for ii, aa in enumerate@@ comments:
+              comments[ii] = '{date}<br/><img src="/test/guestbook?get_avatar={image}"></img><b>{name}</b> wrote:<blockquote>{comment}</blockquote>'.format(
+                comment = aa.comment,
+                date = aa.date,
+                image = aa.key(),
+                name = aa.user.nickname() if aa.user else 'anonymous',
+                )
+            # self.printerr@@ [aa for aa in comments]
+            self.write@@ 'text/html', self.render('url_test_guestbook', comments = '\n'.join(comments))
+
+        def post(self):
+          data = self.DataGuestbook()
+          if appengine.users.get_current_user(): data.user = appengine.users.get_current_user()
+          data.comment = self.request.get('comment')
+          # self.printerr@@ repr@@ data.comment
+          avatar = self.request.get('avatar')
+          if avatar: data.avatar = appengine.db.Blob@@ appengine.images.resize@@ avatar, 32, 32
+          data.put(); self.redirect('/test/guestbook')
+
+      class url_image(URL):
+        def get(self):
+          if 'get_image' in self.query:
+            img = appengine.memcache.get@@ 'url_test_image'
+            if img: self.write@@ 'image', img
+          else: self.write@@ 'text/html', self.render('url_test_image')
+
+        def post(self): appengine.memcache.set@@ 'url_test_image', appengine.images.resize@@ self.request.get('img'), 256, 256; self.redirect('/test/image')
+  import url2; URL.dispatch()
+## END
